#coding=utf8

# This file is part of CloudControl.
#
# CloudControl is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# CloudControl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with CloudControl.  If not, see <http://www.gnu.org/licenses/>.

'''
CloudControl right releated commands
'''

from cccli.command import RemoteCommand
from cccli.exception import *
import os

class RightCommand(RemoteCommand):
    '''Right Base Class'''

    @staticmethod
    def obj2str(obj_rights):
        '''Convert rights object to string'''
        s = ""
        col_len = ""
        for r in obj_rights:
            s += "%(match)s\t%(method)s\t%(tql)s\t%(action)s\n" % r
        return s

    @staticmethod
    def str2obj(str_rights):
        '''Convert rigths string to object'''
        obj = []
        for line in str_rights.split(os.linesep):
            line = line.strip()
            if len(line) == 0 or line.startswith("#"):
                continue
            #print line
            match, method, tql, action = line.split(None, 4)
            obj.append({"match": match,
                        "method": method,
                        "tql": tql,
                        "action": action})
        return obj

    def check_file(self, fo):
        '''Check if fo is valid rights strings'''
        try:
            self.str2obj(fo.read())
        except Exception as e:
            raise
            self.printer.debug("Invalid format: %s" % e)
            return False
        return True


class Command_rights(RightCommand):
    '''List server rights'''

    def __call__(self, argv):
        # ask server
        o_rights = self.rpc.call("loadrights")
        # display answer
        self.printer.out(self.obj2str(o_rights), nl=None)

    @staticmethod
    def remote_functions():
        return set(('loadrights',))


class Command_editrights(RightCommand):
    ''' Edit server rights '''

    def __call__(self, argv):
        obj_rights = self.rpc.call("loadrights")
        str_rights = self.obj2str(obj_rights)
        new_rights = self.printer.editor(str_rights, self.check_file)
        if new_rights is not None:
            self.rpc.call('saverights', self.str2obj(new_rights))
        else:
            self.printer.out("Nothing change.")

    def remote_functions(self):
        return set(('saverights',))
